@@ -2,7 +2,7 @@
 
 /* amfm.{cc,hh} -- Adobe Multiple-Master Font Metrics
  *
- * Copyright (c) 1998-2004 Eddie Kohler
+ * Copyright (c) 1998-2010 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -61,7 +61,7 @@ AmfmMetrics::sanity(ErrorHandler *errh) const
 	errh->error("AMFM sanity: no multiple master interpolation information");
 	return false;
     }
-  
+
     bool ok = true;
     for (int m = 0; m < _nmasters; m++)
 	if (!_masters[m].font_name
@@ -69,10 +69,10 @@ AmfmMetrics::sanity(ErrorHandler *errh) const
 	    errh->error("AMFM sanity: no information for master %d", m);
 	    ok = false;
 	}
-  
+
     if (!_mmspace->check(errh))
 	ok = false;
-  
+
     return ok;
 }
 
@@ -85,7 +85,7 @@ AmfmMetrics::primary_label_value(int ax, PermString label) const
 	if (pf->labels[ax] == label)
 	    return pf->design_vector[ax];
     }
-    return -1;  
+    return -1;
 }
 
 
@@ -99,18 +99,18 @@ Metrics *
 AmfmMetrics::master(int m, ErrorHandler *errh)
 {
     AmfmMaster &master = _masters[m];
-  
+
     if (!master.loaded) {
 	master.loaded = true;
 	DirectoryMetricsFinder directory_finder(_directory);
 	_finder->add_finder(&directory_finder);
 	Metrics *afm = _finder->find_metrics(master.font_name);
-    
+
 	if (!afm) {
 	    if (errh)
 		errh->error("%s: can't find AFM file for master `%s'",
 			    _font_name.c_str(), master.font_name.c_str());
-      
+
 	} else if (!strcompat(afm->font_name(), master.font_name)
 		   || !strcompat(afm->family(), master.family)
 		   || !strcompat(afm->full_name(), master.full_name)
@@ -118,18 +118,18 @@ AmfmMetrics::master(int m, ErrorHandler *errh)
 	    if (errh)
 		errh->error("%s: AFM for master `%s' doesn't match AMFM",
 			    _font_name.c_str(), master.font_name.c_str());
-    
+
 	} else if (!_sanity_afm) {
 	    master.afm = afm;
 	    _sanity_afm = afm;
 	    afm->use();
-      
+
 	} else {
 	    PairProgram *sanity_pairp = _sanity_afm->pair_program();
 	    PairProgram *pairp = afm->pair_program();
 	    char buf[1024];
 	    buf[0] = 0;
-      
+
 	    if (afm->nglyphs() != _sanity_afm->nglyphs())
 		sprintf(buf, "glyph count (%d vs. %d)", afm->nglyphs(), _sanity_afm->nglyphs());
 	    if (afm->nfd() != _sanity_afm->nfd())
@@ -146,7 +146,7 @@ AmfmMetrics::master(int m, ErrorHandler *errh)
 		errh->error("%s: AFM for master `%s' failed sanity checks (%s)", _font_name.c_str(), master.font_name.c_str(), buf);
 	}
     }
-  
+
     return master.afm;
 }
 
@@ -156,12 +156,12 @@ AmfmMetrics::find_primary_font(const Vector<double> &design_vector) const
 {
     assert(design_vector.size() == _naxes);
     for (AmfmPrimaryFont *pf = _primary_fonts; pf; pf = pf->next) {
-    
+
 	for (int a = 0; a < _naxes; a++)
 	    if ((int)design_vector[a] != pf->design_vector[a])
 		goto loser;
 	return pf;
-    
+
       loser: ;
     }
     return 0;
@@ -175,9 +175,9 @@ AmfmMetrics::interpolate(const Vector<double> &design_vector,
 {
     assert(design_vector.size() == _naxes);
     assert(weight_vector.size() == _nmasters);
-  
+
     // FIXME: check masters for correspondence.
-  
+
     /* 0.
      * Make sure all necessary AFMs have been loaded. */
     int m;
@@ -185,10 +185,10 @@ AmfmMetrics::interpolate(const Vector<double> &design_vector,
 	if (weight_vector[m])
 	    if (!master(m, errh))
 		return 0;
-  
+
     /* 1.
      * Use the design vector to generate new FontName and FullName. */
-  
+
     AmfmPrimaryFont *pf = find_primary_font(design_vector);
     // The primary font is useless to us if it doesn't have axis labels.
     if (pf && !pf->labels.size())
@@ -201,7 +201,7 @@ AmfmMetrics::interpolate(const Vector<double> &design_vector,
 	double dv = design_vector[a];
 	font_name_sa << '_' << dv;
 	full_name_sa << (a == 0 ? '_' : ' ') << dv;
-    
+
 	PermString label;
 	if (pf)
 	    label = pf->labels[a];
@@ -215,7 +215,7 @@ AmfmMetrics::interpolate(const Vector<double> &design_vector,
 
     /* 2.
      * Set up the new AFM with the special constructor. */
-  
+
     // Find the first master with a non-zero component.
     for (m = 0; m < _nmasters && weight_vector[m] == 0; m++)
 	;
@@ -224,16 +224,16 @@ AmfmMetrics::interpolate(const Vector<double> &design_vector,
 	AfmMetricsXt *new_xt = new AfmMetricsXt((AfmMetricsXt &)*xt);
 	afm->add_xt(new_xt);
     }
-  
+
     /* 2.
      * Interpolate the old AFM data into the new. */
-  
+
     afm->interpolate_dimens(*_masters[m].afm, weight_vector[m], false);
-  
+
     for (m++; m < _nmasters; m++)
 	if (weight_vector[m])
 	    afm->interpolate_dimens(*_masters[m].afm, weight_vector[m], true);
-  
+
     return afm;
 }
 
@@ -246,7 +246,7 @@ AmfmReader::AmfmReader(AfmParser &afmp, AmfmMetrics *amfm, ErrorHandler *errh)
     : _amfm(amfm), _finder(amfm->_finder), _l(afmp),
       _mmspace(amfm->_mmspace)
 {
-    _errh = errh ? errh : ErrorHandler::ignore_handler();
+    _errh = errh ? errh : ErrorHandler::silent_handler();
 }
 
 AmfmMetrics *
@@ -285,7 +285,7 @@ AmfmReader::lwarning(const char *format, ...) const
 {
     va_list val;
     va_start(val, format);
-    _errh->verror(ErrorHandler::ERR_WARNING, _l.landmark(), format, val);
+    _errh->vxmessage(_l.landmark(), ErrorHandler::e_warning, format, val);
     va_end(val);
 }
 
@@ -294,7 +294,7 @@ AmfmReader::lerror(const char *format, ...) const
 {
     va_list val;
     va_start(val, format);
-    _errh->verror(ErrorHandler::ERR_ERROR, _l.landmark(), format, val);
+    _errh->vxmessage(_l.landmark(), ErrorHandler::e_error, format, val);
     va_end(val);
 }
 
@@ -332,10 +332,10 @@ AmfmReader::read()
 {
     assert(_amfm != 0);
     _mmspace = _amfm->_mmspace;
-  
+
     AfmParser &l = _l;
     _amfm->_directory = l.filename().directory();
-  
+
     // First, read all opening comments into an array so we can print them out
     // later.
     PermString comment;
@@ -349,12 +349,12 @@ AmfmReader::read()
 	    break;
 	}
     }
-  
+
     int master = 0, axis = 0;
-  
+
     while (l.next_line())
 	switch (l[0]) {
-      
+
 	  case 'A':
 	    if (l.isall("Ascender %g", &fd( fdAscender )))
 		break;
@@ -363,7 +363,7 @@ AmfmReader::read()
 		break;
 	    }
 	    goto invalid;
-      
+
 	  case 'B':
 	    if (l.is("BlendDesignPositions")) {
 		read_positions();
@@ -378,26 +378,26 @@ AmfmReader::read()
 		break;
 	    }
 	    goto invalid;
-      
+
 	  case 'C':
 	    if (l.isall("CapHeight %g", &fd( fdCapHeight )))
 		break;
 	    if (l.is("Comment"))
 		break;
 	    goto invalid;
-      
+
 	  case 'D':
 	    if (l.isall("Descender %g", &fd( fdDescender )))
 		break;
 	    goto invalid;
-      
+
 	  case 'E':
 	    if (l.isall("EncodingScheme %+s", &_amfm->_encoding_scheme))
 		break;
 	    if (l.isall("EndMasterFontMetrics"))
 		goto done;
 	    goto invalid;
-      
+
 	  case 'F':
 	    if (l.isall("FontName %+s", &_amfm->_font_name)) {
 		check_mmspace();
@@ -412,26 +412,26 @@ AmfmReader::read()
 			&fd( fdFontBBurx ), &fd( fdFontBBury )))
 		break;
 	    goto invalid;
-      
+
 	  case 'I':
 	    if (l.isall("IsFixedPitch %b", (bool *)0))
 		break;
 	    if (l.isall("ItalicAngle %g", &fd( fdItalicAngle )))
 		break;
 	    goto invalid;
-      
+
 	  case 'M':
 	    if (l.isall("Masters %d", &_amfm->_nmasters)) {
 		check_mmspace();
 		break;
 	    }
 	    goto invalid;
-      
+
 	  case 'N':
 	    if (l.isall("Notice %+s", &_amfm->_notice))
 		break;
 	    goto invalid;
-      
+
 	  case 'S':
 	    if (l.isall("StartAxis")) {
 		read_axis(axis++);
@@ -452,19 +452,19 @@ AmfmReader::read()
 	    if (l.isall("StartMasterFontMetrics %g", (double *)0))
 		break;
 	    goto invalid;
-      
+
 	  case 'U':
 	    if (l.isall("UnderlinePosition %g", &fd( fdUnderlinePosition )))
 		break;
 	    else if (l.isall("UnderlineThickness %g", &fd( fdUnderlineThickness )))
 		break;
 	    goto invalid;
-      
+
 	  case 'V':
 	    if (l.isall("Version %+s", &_amfm->_version))
 		break;
 	    goto invalid;
-      
+
 	  case 'W':
 	    if (l.isall("Weight %+s", &_amfm->_weight))
 		break;
@@ -477,37 +477,37 @@ AmfmReader::read()
 		break;
 	    }
 	    goto invalid;
-      
+
 	  case 'X':
 	    if (l.isall("XHeight %g", &fd( fdXHeight )))
 		break;
 	    goto invalid;
-      
+
 	  default:
 	  invalid:
 	    no_match_warning();
-      
+
 	}
-  
+
   done:
     if (!_mmspace) {
 	_errh->error("`%s' is not an AMFM file", String(_l.landmark().file()).c_str());
 	return false;
     }
-  
+
     LandmarkErrorHandler pin_errh(_errh, _l.landmark());
     if (!_amfm->sanity(&pin_errh)) {
 	_errh->lerror(_l.landmark().whole_file(),
 		      "bad AMFM file (missing or inconsistent information)");
 	return false;
     }
-  
+
     if (!_mmspace->check_intermediate() && _l.filename().directory()) {
 	String name = l.filename().base() + ".amcp";
 	Slurper slurp(_l.filename().from_directory(name));
 	add_amcp_file(slurp, _amfm, _errh);
     }
-  
+
     return true;
 }
 
@@ -516,30 +516,30 @@ void
 AmfmReader::read_amcp_file()
 {
     int lines_read = 0;
-  
+
     while (_l.next_line()) {
 	lines_read++;
 	switch (_l[0]) {
-      
+
 	  case 'C':
 	    if (_l.is("Comment"))
 		break;
 	    goto invalid;
-      
+
 	  case 'S':
 	    if (_l.isall("StartConversionPrograms %d %d", (int *)0, (int *)0)) {
 		read_conversion_programs();
 		break;
 	    }
 	    goto invalid;
-      
+
 	  default:
 	  invalid:
 	    no_match_warning("AMCP file");
-      
+
 	}
     }
-  
+
     if (_mmspace && !_mmspace->ndv() && !_mmspace->cdv() && lines_read)
 	lwarning("no conversion programs in .amcp file");
 }
@@ -549,12 +549,12 @@ bool
 AmfmReader::read_simple_array(Vector<double> &vec) const
 {
     if (!_l.is("[")) return false;
-  
+
     vec.clear();
     double d;
     while (_l.is("%g", &d))
 	vec.push_back(d);
-  
+
     return _l.is("]");
 }
 
@@ -565,17 +565,17 @@ AmfmReader::read_positions() const
     if (nmasters() < 2 || naxes() < 1) return;
     Vector<NumVector> positions;
     if (!_l.is("[") || !_mmspace) goto error;
-  
+
     for (int i = 0; i < nmasters(); i++) {
 	positions.push_back(NumVector());
 	if (!read_simple_array(positions.back()))
 	    goto error;
     }
-  
+
     if (!_l.is("]")) goto error;
     _mmspace->set_master_positions(positions);
     return;
-  
+
   error:
     lerror("bad BlendDesignPositions");
 }
@@ -587,7 +587,7 @@ AmfmReader::read_normalize() const
     if (naxes() < 1) return;
     Vector<NumVector> normalize_in, normalize_out;
     if (!_l.is("[") || !_mmspace) goto error;
-  
+
     for (int a = 0; a < naxes(); a++) {
 	if (!_l.is("[")) goto error;
 	normalize_in.push_back(NumVector());
@@ -599,11 +599,11 @@ AmfmReader::read_normalize() const
 	}
 	if (!_l.is("]")) goto error;
     }
-  
+
     if (!_l.is("]")) goto error;
     _mmspace->set_normalize(normalize_in, normalize_out);
     return;
-  
+
   error:
     lerror("bad BlendDesignPositions");
 }
@@ -621,10 +621,10 @@ AmfmReader::read_axis_types() const
 
     while (_l.is("/%/s", &s))
 	_mmspace->set_axis_type(ax++, s);
-  
+
     if (!_l.is("]")) goto error;
     return;
-  
+
   error:
     lerror("bad BlendAxisTypes");
 }
@@ -638,11 +638,11 @@ AmfmReader::read_axis(int ax) const
 	lerror("bad axis number %d", ax);
     else
 	_mmspace->check();
-  
+
     PermString s;
     while (_l.next_line())
 	switch (_l[0]) {
-      
+
 	  case 'A':
 	    if (_l.is("AxisType %+s", &s)) {
 		if (ok)
@@ -655,23 +655,23 @@ AmfmReader::read_axis(int ax) const
 		break;
 	    }
 	    goto invalid;
-      
+
 	  case 'C':
 	    if (_l.is("Comment"))
 		break;
 	    goto invalid;
-      
+
 	  case 'E':
 	    if (_l.isall("EndAxis"))
 		goto endaxis;
 	    goto invalid;
-      
+
 	  default:
 	  invalid:
 	    no_match_warning();
-      
+
 	}
-  
+
   endaxis: ;
 }
 
@@ -689,21 +689,21 @@ AmfmReader::read_master(int m) const
 	    _amfm->_masters = new AmfmMaster[ nmasters() ];
 	amfmm = &_amfm->_masters[m];
     }
-  
+
     while (_l.next_line())
 	// Grok the whole line. Are we on a character metric data line?
 	switch (_l[0]) {
-      
+
 	  case 'C':
 	    if (_l.is("Comment"))
 		break;
 	    goto invalid;
-      
+
 	  case 'E':
 	    if (_l.isall("EndMaster"))
 		goto endmaster;
 	    goto invalid;
-      
+
 	  case 'F':
 	    if (_l.isall("FontName %+s", &amfmm->font_name))
 		break;
@@ -712,12 +712,12 @@ AmfmReader::read_master(int m) const
 	    if (_l.isall("FamilyName %+s", &amfmm->family))
 		break;
 	    goto invalid;
-      
+
 	  case 'V':
 	    if (_l.isall("Version %+s", &amfmm->version))
 		break;
 	    goto invalid;
-      
+
 	  case 'W':
 	    if (_l.is("WeightVector")) {
 		if (!(read_simple_array(amfmm->weight_vector) &&
@@ -728,13 +728,13 @@ AmfmReader::read_master(int m) const
 		break;
 	    }
 	    goto invalid;
-      
+
 	  default:
 	  invalid:
 	    no_match_warning();
-      
+
 	}
-  
+
   endmaster: ;
 }
 
@@ -745,9 +745,9 @@ AmfmReader::read_one_primary_font() const
     AmfmPrimaryFont *pf = new AmfmPrimaryFont;
     pf->design_vector.resize(naxes());
     pf->labels.resize(naxes());
-  
+
     while (_l.left()) {
-    
+
 	if (_l.is("PC")) {
 	    for (int a = 0; a < naxes(); a++)
 		if (!_l.is("%d", &pf->design_vector[a]))
@@ -760,14 +760,14 @@ AmfmReader::read_one_primary_font() const
 	    ;
 	else
 	    no_match_warning("primary font");
-    
+
 	_l.is(";"); // get rid of any possible semicolon
     }
-  
+
     pf->next = _amfm->_primary_fonts;
     _amfm->_primary_fonts = pf;
     return;
-  
+
   error:
     delete pf;
 }
@@ -778,30 +778,30 @@ AmfmReader::read_primary_fonts() const
 {
     while (_l.next_line())
 	switch (_l[0]) {
-      
+
 	  case 'C':
 	    if (_l.is("Comment"))
 		break;
 	    goto invalid;
-      
+
 	  case 'E':
 	    if (_l.isall("EndPrimaryFonts"))
 		goto end_primary_fonts;
 	    goto invalid;
-      
+
 	  case 'P':
 	    if (_l[1] == 'C' && isspace(_l[2])) {
 		read_one_primary_font();
 		break;
 	    }
 	    goto invalid;
-      
+
 	  default:
 	  invalid:
 	    no_match_warning();
-      
+
 	}
-  
+
   end_primary_fonts: ;
 }
 
@@ -810,36 +810,36 @@ void
 AmfmReader::read_conversion_programs() const
 {
     String ndv, cdv, s;
-  
+
     while (_l.next_line())
 	switch (_l[0]) {
-      
+
 	  case 'C':
 	    if (_l.isall("CDV %<", &s)) {
 		cdv += s;
 		break;
 	    }
 	    goto invalid;
-      
+
 	  case 'E':
 	    if (_l.isall("EndConversionPrograms"))
 		goto end_conversion_programs;
 	    goto invalid;
-      
+
 	  case 'N':
 	    if (_l.isall("NDV %<", &s)) {
 		ndv += s;
 		break;
 	    }
 	    goto invalid;
-      
+
 	  default:
 	  invalid:
 	    no_match_warning();
 	    break;
-      
+
 	}
-  
+
   end_conversion_programs:
     if (_mmspace) {
 	_mmspace->set_ndv(Type1Charstring(ndv));
