@@ -1,6 +1,6 @@
 /* kpseinterface.{c,h} -- interface with the kpathsea library
  *
- * Copyright (c) 2003 Eddie Kohler
+ * Copyright (c) 2003-2010 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -56,6 +56,14 @@ kpsei_find_file(const char* name, int format)
 	return kpse_find_file(name, kpse_tex_ps_header_format, true);
       case KPSEI_FMT_TYPE1:
 	return kpse_find_file(name, kpse_type1_format, false);
+      case KPSEI_FMT_TYPE42:
+	return kpse_find_file(name, kpse_type42_format, false);
+      case KPSEI_FMT_TRUETYPE:
+	return kpse_find_file(name, kpse_truetype_format, false);
+#if HAVE_DECL_KPSE_OPENTYPE_FORMAT
+      case KPSEI_FMT_OPENTYPE:
+	return kpse_find_file(name, kpse_opentype_format, false);
+#endif
       case KPSEI_FMT_OTHER_TEXT:
 	return kpse_find_file(name, kpse_program_text_format, true);
       case KPSEI_FMT_MAP:
