@@ -1,6 +1,6 @@
 /* secondary.{cc,hh} -- code for generating fake glyphs
  *
- * Copyright (c) 2003-2006 Eddie Kohler
+ * Copyright (c) 2003-2010 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -82,7 +82,8 @@ enum { U_EXCLAMDOWN = 0x00A1,	// U+00A1 INVERTED EXCLAMATION MARK
 
 FontInfo::FontInfo(const Efont::OpenType::Font *otf_, ErrorHandler *errh)
     : otf(otf_), cmap(0), cff_file(0), cff(0), post(0), name(0), _nglyphs(-1),
-      _got_glyph_names(false), _ttb_program(0)
+      _got_glyph_names(false), _ttb_program(0), _override_is_fixed_pitch(false),
+      _override_italic_angle(false)
 {
     cmap = new Efont::OpenType::Cmap(otf->table("cmap"), errh);
     assert(cmap->ok());
@@ -110,7 +111,7 @@ FontInfo::FontInfo(const Efont::OpenType::Font *otf_, ErrorHandler *errh)
 	if (_nglyphs < 0 && post->ok())
 	    _nglyphs = post->nglyphs();
     }
-    
+
     name = new Efont::OpenType::Name(otf->table("name"), errh);
 }
 
@@ -190,7 +191,9 @@ FontInfo::program() const
 bool
 FontInfo::is_fixed_pitch() const
 {
-    if (cff) {
+    if (_override_is_fixed_pitch)
+	return _is_fixed_pitch;
+    else if (cff) {
 	double d;
 	return (cff->dict_value(Efont::Cff::oIsFixedPitch, &d) && d);
     } else
@@ -200,7 +203,9 @@ FontInfo::is_fixed_pitch() const
 double
 FontInfo::italic_angle() const
 {
-    if (cff) {
+    if (_override_italic_angle)
+	return _italic_angle;
+    else if (cff) {
 	double d;
 	(void) cff->dict_value(Efont::Cff::oItalicAngle, &d);
 	return d;
@@ -231,7 +236,9 @@ Secondary::encode_uni(int code, PermString name, uint32_t uni, Metrics &metrics,
 T1Secondary::T1Secondary(const FontInfo &finfo, const String &font_name,
 			 const String &otf_file_name)
     : _finfo(finfo), _font_name(font_name), _otf_file_name(otf_file_name),
-      _xheight(font_x_height(finfo, Transform())), _spacewidth(1000)
+      _units_per_em(finfo.units_per_em()),
+      _xheight(font_x_height(finfo, Transform())),
+      _spacewidth(_units_per_em)
 {
     int bounds[4], width;
     if (char_bounds(bounds, width, finfo, Transform(), ' '))
@@ -261,7 +268,7 @@ T1Secondary::char_setting(Vector<Setting> &v, Metrics &metrics, int uni, ...)
 	kerntype = Setting::KERNX;
 	uni = va_arg(val, int);
     }
-    
+
     for (; uni; uni = va_arg(val, int)) {
 	int code = metrics.unicode_encoding(uni);
 	if (code < 0) {
@@ -325,14 +332,14 @@ T1Secondary::dotlessj_font(Metrics &metrics, ErrorHandler *errh, Glyph &dj_glyph
     for (int i = 0; i < metrics.n_mapped_fonts(); i++)
 	if (metrics.mapped_font_name(i) == dj_name)
 	    return i;
-    
+
     if (String filename = installed_type1_dotlessj(_otf_file_name, _finfo.cff->font_name(), (output_flags & G_DOTLESSJ), errh)) {
 
 	// check for special case: "\0" means the font's "j" is already
 	// dotless
 	if (filename == String("\0", 1))
 	    return J_NODOT;
-	
+
 	// open dotless-j font file
 	FILE *f = fopen(filename.c_str(), "rb");
 	if (!f) {
@@ -348,9 +355,9 @@ T1Secondary::dotlessj_font(Metrics &metrics, ErrorHandler *errh, Glyph &dj_glyph
 	    reader = new Efont::Type1PFBReader(f);
 	else
 	    reader = new Efont::Type1PFAReader(f);
-  	Efont::Type1Font *font = new Efont::Type1Font(*reader);
+	Efont::Type1Font *font = new Efont::Type1Font(*reader);
 	delete reader;
-	
+
 	if (!font->ok()) {
 	    errh->error("%s: no glyphs in dotless-J font", filename.c_str());
 	    delete font;
@@ -362,12 +369,12 @@ T1Secondary::dotlessj_font(Metrics &metrics, ErrorHandler *errh, Glyph &dj_glyph
 	font->glyph_names(glyph_names);
 	Vector<PermString>::iterator g = std::find(glyph_names.begin(), glyph_names.end(), "uni0237");
 	if (g == glyph_names.end()) {
-	    errh->error("%s: dotless-J font has no 'uni0237' glyph", filename.c_str());
+	    errh->error("%s: dotless-J font has no %<uni0237%> glyph", filename.c_str());
 	    delete font;
 	    return -1;
 	}
 	dj_glyph = g - glyph_names.begin();
-	
+
 	// create metrics for dotless-J
 	if (install_metrics) {
 	    Metrics dj_metrics(font, 256);
@@ -375,11 +382,11 @@ T1Secondary::dotlessj_font(Metrics &metrics, ErrorHandler *errh, Glyph &dj_glyph
 	    ::dotlessj_file_name = filename;
 	    output_metrics(dj_metrics, font->font_name(), -1, _finfo, String(), String(), dj_name, dotlessj_dvips_include, errh);
 	} else if (verbose)
-	    errh->message("using '%s' for dotless-J font metrics", dj_name.c_str());
-	
+	    errh->message("using %<%s%> for dotless-J font metrics", dj_name.c_str());
+
 	// add font to metrics
 	return metrics.add_mapped_font(font, dj_name);
-	
+
     } else
 	return -1;
 }
@@ -390,9 +397,9 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
     Transform xform;
     int vsize = v.size();
     extern int letterspace;
-    
+
     switch (uni) {
-	
+
       case U_CWM:
       case U_ALTSELECTOR:
 	v.push_back(Setting(Setting::RULE, 0, _xheight));
@@ -506,7 +513,7 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	    return 1;
 	}
 	break;
-	
+
       case U_BARDBL:
 	if (char_setting(v, metrics, '|', 0)) {
 	    if (!_finfo.is_fixed_pitch()) {
@@ -543,7 +550,7 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	    return 1;
 	}
 	break;
-	
+
       case U_THREEQUARTERSEMDASH:
 	if (char_setting(v, metrics, U_ENDASH, 0)) {
 	    if (!_finfo.is_fixed_pitch()) {
@@ -554,13 +561,13 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	    return 1;
 	}
 	break;
-	
+
       case U_CENTIGRADE:
         // TODO: set italic correction to that of a 'C'
 	if (char_setting(v, metrics, U_USE_KERNX, U_DEGREE, 'C', 0))
 	    return 1;
 	break;
-	
+
       case U_INTERROBANG: {
 	  double exclam_offset =
 	      (char_one_bound(_finfo, xform, 4, true, 0, '?', 0)
@@ -595,14 +602,14 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	break;
 
       case U_RINGFITTED: {
-	  int A_width = char_one_bound(_finfo, xform, 4, true, -1000, 'A', 0);
+	  int A_width = char_one_bound(_finfo, xform, 4, true, -_units_per_em, 'A', 0);
 	  uint32_t ring_char = U_RINGABOVE;
-	  int ring_width = char_one_bound(_finfo, xform, 4, true, -1000, ring_char, 0);
-	  if (ring_width <= -1000) {
+	  int ring_width = char_one_bound(_finfo, xform, 4, true, -_units_per_em, ring_char, 0);
+	  if (ring_width <= -_units_per_em) {
 	      ring_char = U_COMBININGRINGABOVE;
-	      ring_width = char_one_bound(_finfo, xform, 4, true, -1000, ring_char, 0);
+	      ring_width = char_one_bound(_finfo, xform, 4, true, -_units_per_em, ring_char, 0);
 	  }
-	  if (A_width > -1000 && ring_width > -1000) {
+	  if (A_width > -_units_per_em && ring_width > -_units_per_em) {
 	      int offset = (A_width - ring_width) / 2;
 	      v.push_back(Setting(Setting::MOVE, offset, 0));
 	      if (char_setting(v, metrics, ring_char, 0)) {
@@ -612,7 +619,7 @@ T1Secondary::setting(uint32_t uni, Vector<Setting> &v, Metrics &metrics, ErrorHa
 	  }
 	  break;
       }
-	
+
     }
 
     // didn't find a good setting, restore v to pristine state
@@ -663,7 +670,7 @@ font_x_height(const FontInfo &finfo, const Transform &font_xform)
 	return os2.x_height();
     } catch (Efont::OpenType::Bounds) {
 	// XXX what if 'x', 'm', 'z' were subject to substitution?
-	return char_one_bound(finfo, font_xform, 3, false, 1000,
+	return char_one_bound(finfo, font_xform, 3, false, finfo.units_per_em(),
 			      'x', 'm', 'z', 0);
     }
 }
@@ -676,7 +683,7 @@ font_cap_height(const FontInfo &finfo, const Transform &font_xform)
 	return os2.cap_height();
     } catch (Efont::OpenType::Bounds) {
 	// XXX what if 'H', 'O', 'B' were subject to substitution?
-	return char_one_bound(finfo, font_xform, 3, false, 1000,
+	return char_one_bound(finfo, font_xform, 3, false, finfo.units_per_em(),
 			      'H', 'O', 'B', 0);
     }
 }
